@@ -198,7 +198,7 @@ func main() {
 	LogLock = os.Getenv(EnvLogLock) == "1"
 	LogJS = os.Getenv(EnvLogJS) == "1"
 	if Debug || os.Getenv(EnvLogPid) == "1" {
-		MyPid = os.Getpid()
+		MyPID = os.Getpid()
 	}
 	var traced bool
 	if flagTraceAll != nil && *flagTraceAll {
