@@ -21,18 +21,13 @@ import (
 	"io"
 	"os"
 	"path"
-	"path/filepath"
 	"strings"
 )
 
 func targetsCollect(root string, tgts map[string]struct{}) error {
-	root, err := filepath.Abs(root)
-	if err != nil {
-		panic(err)
-	}
 	dir, err := os.Open(root)
 	if err != nil {
-		return err
+		return ErrLine(err)
 	}
 	defer dir.Close()
 	for {
@@ -41,7 +36,7 @@ func targetsCollect(root string, tgts map[string]struct{}) error {
 			if err == io.EOF {
 				break
 			}
-			return err
+			return ErrLine(err)
 		}
 		for _, entry := range entries {
 			if !entry.IsDir() {
@@ -51,12 +46,12 @@ func targetsCollect(root string, tgts map[string]struct{}) error {
 			if entry.Name() == RedoDir {
 				redoDir, err := os.Open(pth)
 				if err != nil {
-					return err
+					return ErrLine(err)
 				}
 				redoEntries, err := redoDir.ReadDir(0)
 				redoDir.Close()
 				if err != nil {
-					return err
+					return ErrLine(err)
 				}
 				for _, redoEntry := range redoEntries {
 					name := redoEntry.Name()
@@ -76,9 +71,9 @@ func targetsCollect(root string, tgts map[string]struct{}) error {
 }
 
 func targetsWalker(tgts []string) ([]string, error) {
-	tgtsMap := map[string]struct{}{}
+	tgtsMap := make(map[string]struct{})
 	for _, tgt := range tgts {
-		if err := targetsCollect(tgt, tgtsMap); err != nil {
+		if err := targetsCollect(mustAbs(tgt), tgtsMap); err != nil {
 			return nil, err
 		}
 	}
@@ -90,12 +85,15 @@ func targetsWalker(tgts []string) ([]string, error) {
 }
 
 func collectWholeDeps(
-	tgts map[string]struct{},
-	deps map[string]map[string]struct{},
-	seen map[string]struct{},
+	tgts map[string]*Tgt,
+	deps map[string]map[string]*Tgt,
+	seen map[string]*Tgt,
 ) {
-	for tgt := range tgts {
-		seen[tgt] = struct{}{}
-		collectWholeDeps(deps[tgt], deps, seen)
+	for _, tgt := range tgts {
+		if _, exists := seen[tgt.rel]; exists {
+			continue
+		}
+		seen[tgt.rel] = tgt
+		collectWholeDeps(deps[tgt.rel], deps, seen)
 	}
 }
